@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2020 Richard Hughes <richard@hughsie.com>
+ * Copyright (C) 2017 Richard Hughes <richard@hughsie.com>
  *
  * SPDX-License-Identifier: LGPL-2.1+
  */
@@ -16,11 +16,7 @@ G_DECLARE_DERIVABLE_TYPE (FuHidDevice, fu_hid_device, FU, HID_DEVICE, FuUsbDevic
 struct _FuHidDeviceClass
 {
 	FuUsbDeviceClass parent_class;
-	gboolean	 (*open)		(FuHidDevice		*device,
-						 GError			**error);
-	gboolean	 (*close)		(FuHidDevice		*device,
-						 GError			**error);
-	gpointer	__reserved[29];
+	gpointer	__reserved[31];
 };
 
 /**
@@ -41,6 +37,7 @@ typedef enum {
 	FU_HID_DEVICE_FLAG_RETRY_FAILURE	= 1 << 2,
 	FU_HID_DEVICE_FLAG_NO_KERNEL_UNBIND	= 1 << 3,
 	FU_HID_DEVICE_FLAG_NO_KERNEL_REBIND	= 1 << 4,
+	/*< private >*/
 	FU_HID_DEVICE_FLAG_LAST
 } FuHidDeviceFlags;
 
@@ -56,11 +53,13 @@ gboolean	 fu_hid_device_set_report		(FuHidDevice	*self,
 							 gsize		 bufsz,
 							 guint		 timeout,
 							 FuHidDeviceFlags flags,
-							 GError		**error);
+							 GError		**error)
+							 G_GNUC_WARN_UNUSED_RESULT;
 gboolean	 fu_hid_device_get_report		(FuHidDevice	*self,
 							 guint8		 value,
 							 guint8		*buf,
 							 gsize		 bufsz,
 							 guint		 timeout,
 							 FuHidDeviceFlags flags,
-							 GError		**error);
+							 GError		**error)
+							 G_GNUC_WARN_UNUSED_RESULT;
